/**
 * Copyright (c) 2014 - 2018, David Duan
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form, except as embedded into a Nordic
 *    Semiconductor ASA integrated circuit in a product or a software update for
 *    such product, must reproduce the above copyright notice, this list of
 *    conditions and the following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 * 
 * 3. Neither the name of Nordic Semiconductor ASA nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 * 
 * 4. This software, with or without modification, must only be used with a
 *    Nordic Semiconductor ASA integrated circuit.
 * 
 * 5. Any software provided in binary form under this license must not be reverse
 *    engineered, decompiled, modified and/or disassembled.
 * 
 * THIS SOFTWARE IS PROVIDED BY David Duan "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */
#include <stdbool.h>
#include <stdio.h>
#include <stdint.h>
#include <string.h>

#include "nordic_common.h"
#include "app_error.h"
#include "app_uart.h"
#include "ble.h"
#include "ble_hci.h"
#include "ble_srv_common.h"
#include "ble_advdata.h"
#include "ble_advertising.h"
#include "ble_conn_params.h"
#include "nrf_sdh.h"
#include "nrf_sdh_soc.h"
#include "nrf_sdh_ble.h"
#include "app_timer.h"
#include "app_util_platform.h"
#include "fds.h"
#include "peer_manager.h"
#include "bsp_btn_ble.h"
#include "sensorsim.h"
#include "ble_conn_state.h"
#include "ble_nus.h"
#include "nrf.h"
#include "nrf_sdm.h"
#include "nrf_uart.h"
#include "nrf_drv_timer.h"
#include "nrf_ble_gatt.h"
#include "nrf_ble_qwr.h"
#include "nrf_pwr_mgmt.h"
#include "nrf_drv_saadc.h"
#include "nrf_drv_timer.h"
#include "nrf_drv_ppi.h"
#include "nrf_delay.h"

#include "nrf_soc.h"
#include "nrf_fstorage.h"
#include "nrf_fstorage_sd.h"

#include "nrf_log.h"
#include "nrf_log_ctrl.h"
#include "nrf_log_default_backends.h"

//for bluetooth passkey
#define STATIC_PASSKEY                     "123455" 
//#define STATIC_PASSKEY                  "123455221212321321321321321"                        //static password
//for bluetooth passkey


//for bluetooth advertise data
#define APP_COMPANY_IDENTIFIER           0x4341                          //ASC 0X4341 CA  
#define NUS_SERVICE_UUID_TYPE           BLE_UUID_TYPE_VENDOR_BEGIN                  /**< UUID type for the Nordic UART Service (vendor specific). */
#define APP_ADV_INTERVAL                 300                                     /**< The advertising interval (in units of 0.625 ms. This value corresponds to 187.5 ms). */
#define BLE_ADV_FAST_ENABLED             true

//for bluetooth advertise data

//for saadc
#define SAMPLES_IN_BUFFER 3
//for saadc

//Test command define
#define TestRead_SerialNum                0x01
#define TestRead_SoftwareVer              0x02
#define TestRead_PartNum1                 0x03
#define TestRead_PartNum2                 0x04
#define TestRead_RunningStatus            0x05
#define TestRead_PRE_TestStatus           0x06
#define TestRead_Fnl_TestStatus           0x07
#define TestRead_EOL_Test                 0xA8
#define MAC_Address_Read                  0xA6
#define TestWrite_EOL_Test                0xA7
#define TestWrite_SerialNum               0xA4
#define TestWrite_SoftwareVer             0xA3
#define TestWrite_PartNum1                0xA1
#define TestWrite_PartNum2                0xA2
#define TestWrite_RunningStatus           0x50
#define TestWrite_PRE_TestStatus          0xA5
#define TestWrite_Fnl_TestStatus          0xA0
#define TestEnable_LED_ON                 0x08
#define TestEnable_LED_OFF                0x09
#define TestWrite_Softver_SerNum_PRETest  0xB0
#define TestWrite_FinalT_PN1_PN2          0xB1               
#define Erase_PAGE1                       0xF0 //  ERASE software number,Serialnumber  &  Pre_Test_Status
#define Erase_PAGE2                       0xF1 //  ERASE part number1, part number2 & Final_Test_Status
//Test command define

//flash
#define  FsstartAdd  0x76000
#define  FsendAdd   0x77fff
#define  FlasePage1 0x76000
#define  FlasePage2 0x77000
//flash


//for pin cfg
#define LED_1           17
#define LED_2           18
#define HV_EN           9
#define SAADC_IN_BUFFER 3

#define LED_Green_OFF      nrf_gpio_pin_clear(LED_1)
#define LED_Green_ON       nrf_gpio_pin_set(LED_1)
#define LED_Red_OFF        nrf_gpio_pin_clear(LED_2)
#define LED_Red_ON         nrf_gpio_pin_set(LED_2)
#define HV_EN_ON           nrf_gpio_pin_set(HV_EN) 
#define HV_EN_OFF          nrf_gpio_pin_clear(HV_EN) 
//for pin cfg

//for peer, adv init define
#define APP_ADV_DURATION                0                                       /**<advertisning for loop **/
#define APP_BLE_OBSERVER_PRIO           3                                       /**< Application's BLE observer priority. You shouldn't need to modify this value. */
#define APP_BLE_CONN_CFG_TAG            1                                       /**< A tag identifying the SoftDevice BLE configuration. */

#define MIN_CONN_INTERVAL               MSEC_TO_UNITS(100, UNIT_1_25_MS)        /**< Minimum acceptable connection interval (0.1 seconds). */
#define MAX_CONN_INTERVAL               MSEC_TO_UNITS(200, UNIT_1_25_MS)        /**< Maximum acceptable connection interval (0.2 second). */
#define SLAVE_LATENCY                   0                                       /**< Slave latency. */
#define CONN_SUP_TIMEOUT                MSEC_TO_UNITS(4000, UNIT_10_MS)         /**< Connection supervisory timeout (4 seconds). */

#define FIRST_CONN_PARAMS_UPDATE_DELAY  APP_TIMER_TICKS(5000)                   /**< Time from initiating event (connect or start of notification) to first time sd_ble_gap_conn_param_update is called (5 seconds). */
#define NEXT_CONN_PARAMS_UPDATE_DELAY   APP_TIMER_TICKS(30000)                  /**< Time between each call to sd_ble_gap_conn_param_update after the first call (30 seconds). */
#define MAX_CONN_PARAMS_UPDATE_COUNT    3                                       /**< Number of attempts before giving up the connection parameter negotiation. */

#define SEC_PARAM_BOND                  1                                       /**< Perform  bonding. */
#define SEC_PARAM_MITM                  1                                       /**< Man In The Middle protection  required. The connection need to input the password */
#define SEC_PARAM_LESC                  0                                       /**< LE Secure Connections  enabled. */
#define SEC_PARAM_KEYPRESS              0                                       /**< Keypress notifications not enabled. */
//#define SEC_PARAM_IO_CAPABILITIES       BLE_GAP_IO_CAPS_NONE                    /**< No I/O capabilities. */
#define SEC_PARAM_IO_CAPABILITIES       BLE_GAP_IO_CAPS_DISPLAY_ONLY            /**< Display I/O capabilities. */
#define SEC_PARAM_OOB                   0                                       /**< Out Of Band data not available. */
#define SEC_PARAM_MIN_KEY_SIZE          7                                       /**< Minimum encryption key size. */
#define SEC_PARAM_MAX_KEY_SIZE          16                                      /**< Maximum encryption key size. */

#define DEAD_BEEF                       0xDEADBEEF                              /**< Value used as error code on stack dump, can be used to identify stack location on stack unwind. */

//for peer, adv init define





#define inputOCP_lv0  1000   
#define inputOCP_lv1  1000   
#define inputOCP_lv2  1300   	 
#define inputOCP_lv3  1300   	 
#define inputOCP_lv4  1300   



//for new NTC SDNT1608X473H4150FTF for testing 
#define OTPH_ERT	180	 // 100 deg
#define OTPHR_ERT       243    //  90  deg recover
#define OTPL_ERT	3146	 // -30 deg
#define OTPLR_ERT	3091	 //  -25 deg recover


#define zeor_deg_adc    2561   //zero deg the NTC value
//for threhold define 

//uart 
#define UART_TX_BUF_SIZE                256                                         /**< UART TX buffer size. */
#define UART_RX_BUF_SIZE                256                                         /**< UART RX buffer size. */
//uart

////////////////////////////////////global value///////////////////////////////////

//flash reading 



uint32_t  FWversion =0x76000;
uint32_t  Serialnumber =0x76010;
uint32_t  Pre_Test_Status =0x76020;
uint32_t  Final_Test_Status = 0x77000;
uint32_t  Part_Num1=0x77010;
uint32_t  Part_Num2=0x77020;
uint32_t  EOL_test_status=0x77030;
//flash reading 

// for BLE safety connect
bool    sec_conn = false;                                          //verify this connection is security connection or not 
bool    conn_status = false;                                        //false measning it allow to connect true means only advertise without connectable     
// for BLE safety connect

// for BLE NUS 
static uint16_t m_conn_handle = BLE_CONN_HANDLE_INVALID;                        /**< Handle of the current connection. */
static uint16_t   m_ble_nus_max_data_len = BLE_GATT_ATT_MTU_DEFAULT - 3;            /**< Maximum length of data (in bytes) that can be transmitted to the peer by the Nordic UART service module. */
uint8_t AQS_Rev_Data[7]; 
uint8_t AQS_pack_Rev_Data[12];                                                                 //BLE_Reciving_data
bool  Rec_flag  = false;
// for BLE NUS



uint16_t Temp_val=0;
uint8_t  H_item=0;
uint8_t  L_item=0;
uint16_t  LF_val=5000;
uint16_t  HF_val=5000;
uint16_t  L_diff=0;
uint16_t  H_diff=0;
uint16_t  Final_Temp=0;
uint8_t numitem=0;
// Temp

//for general glabal value
uint8_t powerstatus = 0;  // collect the running status of whole system
bool ONOFF_flag = false;  //output  turn on or off
bool first_flag = true;   //system first turn on or recover
uint8_t powerflag = 0;    //for main loop 
uint8_t ocpcount = 0;     //over current protection count
uint8_t ocpdelaycount=0; //for over current delay count
uint8_t lcpcount = 0;     //low  current protection count
uint8_t otpcount = 0;     //over temp protection count
uint8_t ovpcount = 0;     //over voltage protection count
uint8_t uvpcount = 0;     //under voltage protection count
uint8_t curcount = 0;    //count the current value 
uint32_t current_val=0;   //current read value from ADC
uint16_t Upload_current_val=0; // upload the current value by bluetooth
uint8_t  Pro_level; // level 0 : product mode level 1: production mode level 2: operation mode
//for general glabal value

//for saadc
static const nrf_drv_timer_t m_timer = NRF_DRV_TIMER_INSTANCE(1); //USE Time1 for SAADC & Time count

static nrf_saadc_value_t     m_buffer_pool[2][SAMPLES_IN_BUFFER];
static nrf_ppi_channel_t     m_ppi_channel;
static uint32_t              m_adc_evt_counter;

uint16_t  adc_value[3]; 
//for saadc


//for software ver
uint8_t  softver_value[3]= {0};
//for software ver

// for sec peer
static pm_peer_id_t m_peer_to_be_deleted = PM_PEER_ID_INVALID;
// for sec peer


//for time count 
uint16_t Timecount = 0;
uint16_t Delaycount = 0;
uint16_t unsafe_conn_Time_count=0;                                         //Time count when connection not peer if the over the timecount disconnect the device
uint16_t LED_count=0;
bool adv_flag = false;
//for time count


//logic flag value
bool UVPflag = false;
bool OVPflag = false;
bool OCPflag = false;
bool LCPflag = false;
bool LEDvisFlag = false;   //enable or disenable LED visibility testing 
//logic flag value


//for adv
uint8_t advert_str[24];
uint16_t adv_str_len = 16;      //max length is 21
uint8_t *DEVICE_NAME="CAAQS";                                                           /**< Name of device. Will be included in the advertising data. */
static uint8_t m_adv_handle = BLE_GAP_ADV_SET_HANDLE_NOT_SET;                           /**< Advertising handle used to identify an advertising set. */
static uint8_t m_enc_advdata[BLE_GAP_ADV_SET_DATA_SIZE_MAX];                            /**< Buffer for storing an encoded advertising set. */
static uint8_t m_enc_scan_response_data[BLE_GAP_ADV_SET_DATA_SIZE_MAX];                 /**< Buffer for storing an encoded scan data. */
static volatile bool update_request = false;
//for adv


/* Dummy data to read/write to flash. */
static uint32_t Flash_W_data          = 0XFFFFFFFF;
static uint32_t Flash_R_data          = 0XFFFFFFFF;
static void fstorage_evt_handler(nrf_fstorage_evt_t * p_evt);



//for encapsulation define 
BLE_NUS_DEF(m_nus, NRF_SDH_BLE_TOTAL_LINK_COUNT);                                   /**< BLE NUS service instance. */
NRF_BLE_GATT_DEF(m_gatt);                                                       /**< GATT module instance. */
NRF_BLE_QWR_DEF(m_qwr);                                                         /**< Context for the Queued Write module.*/
//for encapsulation define 

////////////////////////////////////struct define///////////////////////////////////
/**@brief Struct that contains pointers to the encoded advertising data. */
static ble_gap_adv_data_t m_adv_data =
{
    .adv_data =
    {
        .p_data = m_enc_advdata,
        .len    = BLE_GAP_ADV_SET_DATA_SIZE_MAX
    },
    .scan_rsp_data =
    {
        .p_data = m_enc_scan_response_data,
        .len    = BLE_GAP_ADV_SET_DATA_SIZE_MAX

    }
};

//for BLE  sending Data

/**@brief Sending Data Package */
typedef struct
{
  uint8_t   Length;
  uint8_t   Data[7];        /**< Connection Signature Resolving Key. */
} AQS_BLE_Send_Data_Package;

AQS_BLE_Send_Data_Package  AQS_Send_Data;
//for BLE  sending Data

//for adv service ID
static ble_uuid_t m_adv_uuids[] =                                               /**< Universally unique service identifiers. */
{
   // {BLE_UUID_DEVICE_INFORMATION_SERVICE, BLE_UUID_TYPE_BLE}
   {BLE_UUID_NUS_SERVICE, NUS_SERVICE_UUID_TYPE}
};
//for adv service ID


static void advertising_start(bool erase_bonds);

/**@brief Callback function for asserts in the SoftDevice.
 *
 * @details This function will be called in case of an assert in the SoftDevice.
 *
 * @warning This handler is an example only and does not fit a final product. You need to analyze
 *          how your product is supposed to react in case of Assert.
 * @warning On assert from the SoftDevice, the system can only recover on reset.
 *
 * @param[in] line_num   Line number of the failing ASSERT call.
 * @param[in] file_name  File name of the failing ASSERT call.
 */
void assert_nrf_callback(uint16_t line_num, const uint8_t * p_file_name)
{
    app_error_handler(DEAD_BEEF, line_num, p_file_name);
}









/**@brief Function for adc event handle function 
 *
 * @param[out] None
 */
void saadc_callback(nrf_drv_saadc_evt_t const * p_event)
{
   uint16_t adc_read_value;
   uint32_t Red_LED_status;
   uint8_t Green_LED_status;
   ret_code_t err_code;
    //Timercounter
     Timecount++;
     Delaycount++;
     if(Delaycount>=500)
     {
        Delaycount=501;
     }
    //for unsafe connect
    if(Pro_level==2)
    {
      unsafe_conn_Time_count++;
      if(unsafe_conn_Time_count>50000)
      {
          if(sec_conn==false)
          {      
             err_code=sd_ble_gap_disconnect(m_conn_handle,BLE_HCI_REMOTE_USER_TERMINATED_CONNECTION);
             APP_ERROR_CHECK(err_code);
            Pro_level=1;
          }
          unsafe_conn_Time_count=0;
      }
      
    }
    else
    {
      unsafe_conn_Time_count=0;
    }
   


    //saadc
    if (p_event->type == NRF_DRV_SAADC_EVT_DONE)
    {
        ret_code_t err_code;

        err_code = nrf_drv_saadc_buffer_convert(p_event->data.done.p_buffer, SAMPLES_IN_BUFFER);
        APP_ERROR_CHECK(err_code);

        int i;
      //  NRF_LOG_INFO("ADC event number: %d", (int)m_adc_evt_counter);
        adc_value[0]=p_event->data.done.p_buffer[0];
        adc_value[1]=p_event->data.done.p_buffer[1];
        adc_value[2]=p_event->data.done.p_buffer[2];
        
        for (i = 0; i < 3; i++)
          {
              adc_read_value=p_event->data.done.p_buffer[i];
              if(adc_read_value>65530)
              {
                  adc_read_value=0;
              }
              adc_value[i]=adc_read_value*3.5133495145631067961165048543689;
          }

                       
    }
}





/**@brief Function for custizmize borad Digtal I/O , A/D, Time counter setting 
 *
 * @param[out] None
 */

 void bsp_demo_init(void)
{
    ret_code_t err_code;
    nrf_gpio_cfg_output(LED_1);
    nrf_gpio_cfg_output(LED_2);
    nrf_gpio_cfg_output(HV_EN); 

    nrf_gpio_pin_clear(LED_1);
    nrf_gpio_pin_clear(LED_2);
   // nrf_gpio_pin_set(LED_2);
    nrf_gpio_pin_clear(HV_EN);

       
    //ADC for testing 
     nrf_saadc_channel_config_t channel_config1 = NRF_DRV_SAADC_DEFAULT_CHANNEL_CONFIG_SE(NRF_SAADC_INPUT_AIN6);
 //    nrf_saadc_channel_config_t channel_config1 = NRF_DRV_SAADC_DEFAULT_CHANNEL_CONFIG_SE(NRF_SAADC_INPUT_AIN4); // for testing 
     nrf_saadc_channel_config_t channel_config2 = NRF_DRV_SAADC_DEFAULT_CHANNEL_CONFIG_SE(NRF_SAADC_INPUT_AIN2);
     nrf_saadc_channel_config_t channel_config3 =  NRF_DRV_SAADC_DEFAULT_CHANNEL_CONFIG_SE(NRF_SAADC_INPUT_AIN1);

     
    err_code = nrf_drv_saadc_init(NULL, saadc_callback);
    APP_ERROR_CHECK(err_code);
    
    //ADC for testing 
    err_code = nrf_drv_saadc_channel_init(2, &channel_config2);
    APP_ERROR_CHECK(err_code);
   //      err_code = nrf_drv_saadc_channel_init(4, &channel_config1);
  //  APP_ERROR_CHECK(err_code);  
     err_code = nrf_drv_saadc_channel_init(6, &channel_config1);
    APP_ERROR_CHECK(err_code);  
     err_code = nrf_drv_saadc_channel_init(1, &channel_config3);
    APP_ERROR_CHECK(err_code);


    err_code = nrf_drv_saadc_buffer_convert(m_buffer_pool[0], SAMPLES_IN_BUFFER);
    APP_ERROR_CHECK(err_code);

    err_code = nrf_drv_saadc_buffer_convert(m_buffer_pool[1], SAMPLES_IN_BUFFER);
    APP_ERROR_CHECK(err_code);


}

/**@brief   Function for handling app_uart events.
 *
 * @details This function will receive a single character from the app_uart module and append it to
 *          a string. The string will be be sent over BLE when the last character received was a
 *          'new line' '\n' (hex 0x0A) or if the string has reached the maximum data length.
 */
/**@snippet [Handling the data received over UART] */

void uart_event_handle(app_uart_evt_t * p_event)
{
   static uint8_t data_array[BLE_NUS_MAX_DATA_LEN];
   static uint8_t transdata[2];
   static uint8_t index = 0;
   uint32_t       err_code;

    switch (p_event->evt_type)
    {
       case APP_UART_DATA_READY:
            UNUSED_VARIABLE(app_uart_get(&data_array[index]));
            index++;
            if(index > 1)
            {
              NRF_LOG_DEBUG("Ready to send data over BLE NUS");
              NRF_LOG_HEXDUMP_DEBUG(data_array, index);
        
              index = 0;
            }        
            break;
/*
        case APP_UART_COMMUNICATION_ERROR:
            APP_ERROR_HANDLER(p_event->data.error_communication);
            break;

        case APP_UART_FIFO_ERROR:
            APP_ERROR_HANDLER(p_event->data.error_code);
            break;
*/
        default:
            break;
    }

}

/**@brief Function for handling the data from the Nordic UART Service.
 *
 * @details This function will process the data received from the Nordic UART BLE Service and send
 *          it to the UART module.
 *
 * @param[in] p_evt       Nordic UART Service event.
 */
/**@snippet [Handling the data received over BLE] */
static void nus_data_handler(ble_nus_evt_t * p_evt)
{
    if (p_evt->type == BLE_NUS_EVT_RX_DATA)
    {
        uint32_t err_code;
        uint32_t write_data;
        uint32_t write_data1;
        uint32_t write_data2;
        uint32_t write_data3;
   //     NRF_LOG_DEBUG("Received data from BLE NUS. Writing data on UART.");
   //     NRF_LOG_HEXDUMP_DEBUG(p_evt->params.rx_data.p_data, p_evt->params.rx_data.length);
        NRF_LOG_INFO("Received data from BLE NUS. Writing data on UART.");
       // NRF_LOG_INFO("p_evt->type: %d", p_evt->type );
        if( p_evt->params.rx_data.length == 7)
        {       
             NRF_LOG_INFO("p_evt->type: %d", p_evt->params.rx_data.p_data[6] );
           if(((p_evt->params.rx_data.p_data[6] == 0x80)||(p_evt->params.rx_data.p_data[6] == 0x81))&&(p_evt->params.rx_data.p_data[0] == 0xED))
           {
              for(uint8_t i = 0; i< 7;i++)
              {
                AQS_Rev_Data[i]=p_evt->params.rx_data.p_data[i];
              }
              Rec_flag=true;
            
           }     
          
        }
       
    }

}




/**@brief Function for handling Peer Manager events.
 *
 * @param[in] p_evt  Peer Manager event.
 */
 
static void pm_evt_handler(pm_evt_t const * p_evt)
{
    ret_code_t err_code;

    pm_handler_on_pm_evt(p_evt);
    pm_handler_disconnect_on_sec_failure(p_evt);
    pm_handler_flash_clean(p_evt);

    switch (p_evt->evt_id)
    {
      
        case PM_EVT_CONN_SEC_SUCCEEDED:
        {
            sec_conn=true;
            pm_conn_sec_status_t conn_sec_status;
            NRF_LOG_INFO("PM_EVT_CONN_SEC_SUCCEEDED");
            // Check if the link is authenticated (meaning at least MITM).
            err_code = pm_conn_sec_status_get(p_evt->conn_handle, &conn_sec_status);
            APP_ERROR_CHECK(err_code);

            if (conn_sec_status.mitm_protected)
            {
                NRF_LOG_INFO("Link secured. Role: %d. conn_handle: %d, Procedure: %d",
                             ble_conn_state_role(p_evt->conn_handle),
                             p_evt->conn_handle,
                             p_evt->params.conn_sec_succeeded.procedure);
            }
             else
            {
                // The peer did not use MITM, disconnect.
                NRF_LOG_INFO("Collector did not use MITM, disconnecting");
                err_code = pm_peer_id_get(m_conn_handle, &m_peer_to_be_deleted);
                APP_ERROR_CHECK(err_code);
                err_code = sd_ble_gap_disconnect(m_conn_handle,
                                                 BLE_HCI_REMOTE_USER_TERMINATED_CONNECTION);
                APP_ERROR_CHECK(err_code);
            }
        
        } break;

        case PM_EVT_CONN_SEC_CONFIG_REQ:
        {
            NRF_LOG_INFO("PM_EVT_CONN_SEC_CONFIG_REQ");
            // Reject pairing request from an already bonded peer.
            pm_conn_sec_config_t conn_sec_config = {.allow_repairing = true};
            pm_conn_sec_config_reply(p_evt->conn_handle, &conn_sec_config);
        }break;

        case PM_EVT_CONN_SEC_FAILED:
            NRF_LOG_INFO("PM_EVT_CONN_SEC_FAILED");
            m_conn_handle = BLE_CONN_HANDLE_INVALID;
            break;        
    
        case PM_EVT_PEERS_DELETE_SUCCEEDED:
             NRF_LOG_INFO("PM_EVT_PEERS_DELETE_SUCCEEDED");
            Pro_level=1;
            break;
      
        default:
            break;
    }
}


/**@brief Function for the Timer initialization.
 *
 * @details Initializes the timer module. This creates and starts application timers.
 */
static void timers_init(void)
{
    // Initialize timer module.
    ret_code_t err_code = app_timer_init();
    APP_ERROR_CHECK(err_code);
 
}



/**@snippet [Handling the data received over UART] */


/**@brief  Function for initializing the UART module.
 */
/**@snippet [UART Initialization] */
static void uart_init(void)
{
    uint32_t                     err_code;
    app_uart_comm_params_t const comm_params =
    {
        .rx_pin_no    = RX_PIN_NUMBER,
        .tx_pin_no    = TX_PIN_NUMBER,
        .rts_pin_no   = RTS_PIN_NUMBER,
        .cts_pin_no   = CTS_PIN_NUMBER,
        .flow_control = APP_UART_FLOW_CONTROL_DISABLED,
        .use_parity   = false,
#if defined (UART_PRESENT)
        .baud_rate    = NRF_UART_BAUDRATE_115200
#else
        .baud_rate    = NRF_UARTE_BAUDRATE_115200
#endif
    };

    APP_UART_FIFO_INIT(&comm_params,
                       UART_RX_BUF_SIZE,
                       UART_TX_BUF_SIZE,
                       uart_event_handle,
                       APP_IRQ_PRIORITY_LOWEST,
                       err_code);
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for the GAP initialization.
 *
 * @details This function sets up all the necessary GAP (Generic Access Profile) parameters of the
 *          device including the device name, appearance, and the preferred connection parameters.
 */
static void gap_params_init(void)
{
    ret_code_t              err_code;
    ble_gap_conn_params_t   gap_conn_params;
    ble_gap_conn_sec_mode_t sec_mode;

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&sec_mode);

    err_code = sd_ble_gap_device_name_set(&sec_mode,
                                          (const uint8_t *)DEVICE_NAME,
                                          strlen(DEVICE_NAME));
    APP_ERROR_CHECK(err_code);

    memset(&gap_conn_params, 0, sizeof(gap_conn_params));

    gap_conn_params.min_conn_interval = MIN_CONN_INTERVAL;
    gap_conn_params.max_conn_interval = MAX_CONN_INTERVAL;
    gap_conn_params.slave_latency     = SLAVE_LATENCY;
    gap_conn_params.conn_sup_timeout  = CONN_SUP_TIMEOUT;

    err_code = sd_ble_gap_ppcp_set(&gap_conn_params);
    APP_ERROR_CHECK(err_code);

  
    
}


/**@brief Function for handling events from the GATT library. */
void gatt_evt_handler(nrf_ble_gatt_t * p_gatt, nrf_ble_gatt_evt_t const * p_evt)
{
    if ((m_conn_handle == p_evt->conn_handle) && (p_evt->evt_id == NRF_BLE_GATT_EVT_ATT_MTU_UPDATED))
    {
        m_ble_nus_max_data_len = p_evt->params.att_mtu_effective - OPCODE_LENGTH - HANDLE_LENGTH;
        NRF_LOG_INFO("Data len is set to 0x%X(%d)", m_ble_nus_max_data_len, m_ble_nus_max_data_len);
    }
    NRF_LOG_DEBUG("ATT MTU exchange completed. central 0x%x peripheral 0x%x",
                  p_gatt->att_mtu_desired_central,
                  p_gatt->att_mtu_desired_periph);
}



/**@brief Function for initializing the GATT module.
 */
static void gatt_init(void)
{
  
   ret_code_t err_code = nrf_ble_gatt_init(&m_gatt, gatt_evt_handler);
   APP_ERROR_CHECK(err_code);

   err_code = nrf_ble_gatt_att_mtu_periph_set(&m_gatt, NRF_SDH_BLE_GATT_MAX_MTU_SIZE);
   APP_ERROR_CHECK(err_code);

}


/**@brief Function for handling Queued Write Module errors.
 *
 * @details A pointer to this function will be passed to each service which may need to inform the
 *          application about an error.
 *
 * @param[in]   nrf_error   Error code containing information about what went wrong.
 */
static void nrf_qwr_error_handler(uint32_t nrf_error)
{
    APP_ERROR_HANDLER(nrf_error);
}



/**@brief Function for initializing services that will be used by the application.
 */
static void services_init(void)
{
    ret_code_t         err_code;
    ble_nus_init_t     nus_init;
    nrf_ble_qwr_init_t qwr_init = {0};

    // Initialize Queued Write Module.
    qwr_init.error_handler = nrf_qwr_error_handler;

    err_code = nrf_ble_qwr_init(&m_qwr, &qwr_init);
    APP_ERROR_CHECK(err_code);
    
    // Initialize Glucose Service - sample selection of feature bits.
  //  memset(&nus_init, 0, sizeof(nus_init));

    nus_init.data_handler = nus_data_handler;
    err_code = ble_nus_init(&m_nus, &nus_init);
    APP_ERROR_CHECK(err_code);
 
    
}


/**@brief Function for handling the Connection Parameters Module.
 *
 * @details This function will be called for all events in the Connection Parameters Module which
 *          are passed to the application.
 *          @note All this function does is to disconnect. This could have been done by simply
 *                setting the disconnect_on_fail config parameter, but instead we use the event
 *                handler mechanism to demonstrate its use.
 *
 * @param[in] p_evt  Event received from the Connection Parameters Module.
 */
static void on_conn_params_evt(ble_conn_params_evt_t * p_evt)
{
    ret_code_t err_code;

    if (p_evt->evt_type == BLE_CONN_PARAMS_EVT_FAILED)
    {
        err_code = sd_ble_gap_disconnect(m_conn_handle, BLE_HCI_CONN_INTERVAL_UNACCEPTABLE);
        APP_ERROR_CHECK(err_code);
    }
}


/**@brief Function for handling a Connection Parameters error.
 *
 * @param[in] nrf_error  Error code containing information about what went wrong.
 */
static void conn_params_error_handler(uint32_t nrf_error)
{
    APP_ERROR_HANDLER(nrf_error);
}


/**@brief Function for initializing the Connection Parameters module.
 */
static void conn_params_init(void)
{
    ret_code_t             err_code;
    ble_conn_params_init_t cp_init;

    memset(&cp_init, 0, sizeof(cp_init));

    cp_init.p_conn_params                  = NULL;
    cp_init.first_conn_params_update_delay = FIRST_CONN_PARAMS_UPDATE_DELAY;
    cp_init.next_conn_params_update_delay  = NEXT_CONN_PARAMS_UPDATE_DELAY;
    cp_init.max_conn_params_update_count   = MAX_CONN_PARAMS_UPDATE_COUNT;
    cp_init.start_on_notify_cccd_handle    = BLE_GATT_HANDLE_INVALID;
    cp_init.disconnect_on_fail             = false;
    cp_init.evt_handler                    = on_conn_params_evt;
    cp_init.error_handler                  = conn_params_error_handler;

    err_code = ble_conn_params_init(&cp_init);
    APP_ERROR_CHECK(err_code);
}




/**@brief Function for putting the chip into sleep mode.
 *
 * @note This function will not return.
 */
static void sleep_mode_enter(void)
{
    ret_code_t err_code;

    err_code = bsp_indication_set(BSP_INDICATE_IDLE);
    APP_ERROR_CHECK(err_code);

    // Prepare wakeup buttons.
    err_code = bsp_btn_ble_sleep_mode_prepare();
    APP_ERROR_CHECK(err_code);

    // Go to system-off mode (this function will not return; wakeup will cause a reset).
    err_code = sd_power_system_off();
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for handling advertising events.
 *
 * @details This function will be called for advertising events which are passed to the application.
 *
 * @param[in] ble_adv_evt  Advertising event.
 */
static void on_adv_evt(ble_adv_evt_t ble_adv_evt)
{
    ret_code_t err_code;

    switch (ble_adv_evt)
    {
        case BLE_ADV_EVT_FAST:
            NRF_LOG_INFO("Fast advertising.");
            err_code = bsp_indication_set(BSP_INDICATE_ADVERTISING);
            APP_ERROR_CHECK(err_code);
            break;

        case BLE_ADV_EVT_IDLE:
            sleep_mode_enter();
            break;

        case BLE_ADV_MODE_FAST_NOCONNECT:
             NRF_LOG_INFO("Fast advertising.");
             err_code = bsp_indication_set(BSP_INDICATE_ADVERTISING);
             APP_ERROR_CHECK(err_code);
             break;
        default:
            break;
    }
}


/**@brief Function for handling BLE events.
 *
 * @param[in]   p_ble_evt   Bluetooth stack event.
 * @param[in]   p_context   Unused.
 */
static void ble_evt_handler(ble_evt_t const * p_ble_evt, void * p_context)
{
    ret_code_t err_code = NRF_SUCCESS;
    uint16_t role        = ble_conn_state_role(m_conn_handle);
/*
     if (role == BLE_GAP_ROLE_PERIPH)
     {
         NRF_LOG_INFO("PERL connecting");
     }
     else
     {
        NRF_LOG_INFO("CENTRL connecting");
     }
     */
    switch (p_ble_evt->header.evt_id)
    {
      case BLE_GAP_EVT_CONNECTED:
            NRF_LOG_INFO("Connected");
         //   m_peer_to_be_deleted = PM_PEER_ID_INVALID;
            m_conn_handle = p_ble_evt->evt.gap_evt.conn_handle;
            err_code = nrf_ble_qwr_conn_handle_assign(&m_qwr, m_conn_handle);
            APP_ERROR_CHECK(err_code);
            conn_status=true;
            sec_conn=false; // unsafe connection
            break;
     case BLE_GAP_EVT_DISCONNECTED:
            NRF_LOG_INFO("Disconnected");
            // LED indication will be changed when advertising starts.
            sec_conn=false;
            m_conn_handle = BLE_CONN_HANDLE_INVALID;
            if (m_peer_to_be_deleted != PM_PEER_ID_INVALID)
            {
                err_code = pm_peer_delete(m_peer_to_be_deleted);
                APP_ERROR_CHECK(err_code);
                NRF_LOG_DEBUG("Collector's bond deleted");
                 printf("\r\Collector's bond deleted.\r\n");
                m_peer_to_be_deleted = PM_PEER_ID_INVALID;
            }
            
            sec_conn=false;
            conn_status=false;
            break;

       case BLE_GAP_EVT_PHY_UPDATE_REQUEST:
        {
            NRF_LOG_DEBUG("PHY update request.");
            ble_gap_phys_t const phys =
            {
                .rx_phys = BLE_GAP_PHY_AUTO,
                .tx_phys = BLE_GAP_PHY_AUTO,
            };
            err_code = sd_ble_gap_phy_update(p_ble_evt->evt.gap_evt.conn_handle, &phys);
            APP_ERROR_CHECK(err_code);
        } break;

        case BLE_GAP_EVT_SEC_PARAMS_REQUEST:
             NRF_LOG_DEBUG("BLE_GAP_EVT_SEC_PARAMS_REQUEST");
            /*
            err_code = sd_ble_gap_sec_params_reply(m_conn_handle, BLE_GAP_SEC_STATUS_SUCCESS, NULL, NULL);
            APP_ERROR_CHECK(err_code); // the error point
            */
            break;

        case BLE_GATTS_EVT_SYS_ATTR_MISSING:
            // No system attributes have been stored.
            err_code = sd_ble_gatts_sys_attr_set(m_conn_handle, NULL, 0, 0);
            APP_ERROR_CHECK(err_code);
            break;
        case BLE_GATTC_EVT_TIMEOUT:
            // Disconnect on GATT Client timeout event.
            err_code = sd_ble_gap_disconnect(p_ble_evt->evt.gattc_evt.conn_handle,
                                             BLE_HCI_REMOTE_USER_TERMINATED_CONNECTION);
            APP_ERROR_CHECK(err_code);
            break;

        case BLE_GATTS_EVT_TIMEOUT:
            // Disconnect on GATT Server timeout event.
            err_code = sd_ble_gap_disconnect(p_ble_evt->evt.gatts_evt.conn_handle,
                                             BLE_HCI_REMOTE_USER_TERMINATED_CONNECTION);
            APP_ERROR_CHECK(err_code);
            break;
        case BLE_GAP_EVT_PASSKEY_DISPLAY:
        
          NRF_LOG_INFO("BLE_GAP_EVT_PASSKEY_DISPLAY.");
          NRF_LOG_INFO("Passkey: %s", p_ble_evt->evt.gap_evt.params.passkey_display.passkey);
             break;
        
        case BLE_GAP_EVT_AUTH_KEY_REQUEST:
            NRF_LOG_INFO("BLE_GAP_EVT_AUTH_KEY_REQUEST");
            break;

        case BLE_GAP_EVT_LESC_DHKEY_REQUEST:
            NRF_LOG_INFO("BLE_GAP_EVT_LESC_DHKEY_REQUEST");
            break;

         case BLE_GAP_EVT_AUTH_STATUS:
             sec_conn = true;
             printf("\r\nAUTH_STATUS.\r\n");
             NRF_LOG_INFO("BLE_GAP_EVT_AUTH_STATUS: status=0x%x bond=0x%x lv4: %d kdist_own:0x%x kdist_peer:0x%x",
                          p_ble_evt->evt.gap_evt.params.auth_status.auth_status,
                          p_ble_evt->evt.gap_evt.params.auth_status.bonded,
                          p_ble_evt->evt.gap_evt.params.auth_status.sm1_levels.lv4,
                          *((uint8_t *)&p_ble_evt->evt.gap_evt.params.auth_status.kdist_own),
                          *((uint8_t *)&p_ble_evt->evt.gap_evt.params.auth_status.kdist_peer));
            break;
        default:
            // No implementation needed.
         break;
    }
    
}


/**@brief Function for initializing the BLE stack.
 *
 * @details Initializes the SoftDevice and the BLE event interrupt.
 */
static void ble_stack_init(void)
{
    ret_code_t err_code;

    err_code = nrf_sdh_enable_request();
    APP_ERROR_CHECK(err_code);

    // Configure the BLE stack using the default settings.
    // Fetch the start address of the application RAM.
    uint32_t ram_start = 0;
    err_code = nrf_sdh_ble_default_cfg_set(APP_BLE_CONN_CFG_TAG, &ram_start);
    APP_ERROR_CHECK(err_code);

    // Enable BLE stack.
    err_code = nrf_sdh_ble_enable(&ram_start);
    APP_ERROR_CHECK(err_code);

    // Register a handler for BLE events.
    NRF_SDH_BLE_OBSERVER(m_ble_observer, APP_BLE_OBSERVER_PRIO, ble_evt_handler, NULL);
}


/**@brief Function for the Peer Manager initialization.
 */
 
static void peer_manager_init(void)
{
    ble_gap_sec_params_t sec_param;
    ret_code_t           err_code;

    err_code = pm_init();
    APP_ERROR_CHECK(err_code);

    memset(&sec_param, 0, sizeof(ble_gap_sec_params_t));

    // Security parameters to be used for all security procedures.
    sec_param.bond           = SEC_PARAM_BOND;
    sec_param.mitm           = SEC_PARAM_MITM;
    sec_param.lesc           = SEC_PARAM_LESC;
    sec_param.keypress       = SEC_PARAM_KEYPRESS;
    sec_param.io_caps        = SEC_PARAM_IO_CAPABILITIES;
    sec_param.oob            = SEC_PARAM_OOB;
    sec_param.min_key_size   = SEC_PARAM_MIN_KEY_SIZE;
    sec_param.max_key_size   = SEC_PARAM_MAX_KEY_SIZE;
    
    sec_param.kdist_own.enc  = 1;
    sec_param.kdist_own.id   = 1;
    sec_param.kdist_peer.enc = 1;
    sec_param.kdist_peer.id  = 1;

    err_code = pm_sec_params_set(&sec_param);
    APP_ERROR_CHECK(err_code);

    err_code = pm_register(pm_evt_handler);
    APP_ERROR_CHECK(err_code);
}


/**@brief Clear bond information from persistent storage.
 */
static void delete_bonds(void)
{
    ret_code_t err_code;

    NRF_LOG_INFO("Erase bonds!");

    err_code = pm_peers_delete();
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for handling events from the BSP module.
 *
 * @param[in]   event   Event generated when button is pressed.
 */
static void bsp_event_handler(bsp_event_t event)
{
    ret_code_t err_code;

    switch (event)
    {
        case BSP_EVENT_SLEEP:
            sleep_mode_enter();
            break; // BSP_EVENT_SLEEP

        case BSP_EVENT_DISCONNECT:
            err_code = sd_ble_gap_disconnect(m_conn_handle,
                                             BLE_HCI_REMOTE_USER_TERMINATED_CONNECTION);
            if (err_code != NRF_ERROR_INVALID_STATE)
            {
                APP_ERROR_CHECK(err_code);
            }
            break; // BSP_EVENT_DISCONNECT

        case BSP_EVENT_WHITELIST_OFF:
            if (m_conn_handle == BLE_CONN_HANDLE_INVALID)
            {
              //  err_code = ble_advertising_restart_without_whitelist(&m_advertising);
                if (err_code != NRF_ERROR_INVALID_STATE)
                {
                    APP_ERROR_CHECK(err_code);
                }
            }
            break; // BSP_EVENT_KEY_0

        default:
            break;
    }
}



/**@brief Function for initializing the Advertising functionality.
 */
static void advertising_init(void)
{
    ret_code_t    err_code;
    ble_advdata_t advdata;
    ble_advdata_t srdata;
    ble_advdata_manuf_data_t manuf_data;
    ble_gap_adv_params_t adv_params;
    
    ble_uuid_t adv_uuids[] = {{BLE_UUID_NUS_SERVICE, BLE_UUID_TYPE_VENDOR_BEGIN}};
    
    memset(&advdata, 0, sizeof(advdata));

    manuf_data.company_identifier = 0x4341;
    manuf_data.data.p_data        = advert_str;
    manuf_data.data.size          = adv_str_len;

    advdata.name_type             = BLE_ADVDATA_FULL_NAME;
    advdata.include_appearance    = false;
    advdata.flags                 = BLE_GAP_ADV_FLAGS_LE_ONLY_GENERAL_DISC_MODE;
    advdata.p_manuf_specific_data = &manuf_data;

  //  err_code = ble_advdata_encode(&advdata, m_adv_data.adv_data.p_data, &m_adv_data.adv_data.len);
  //  APP_ERROR_CHECK(err_code);
    
    memset(&srdata, 0, sizeof(srdata));
    srdata.uuids_complete.uuid_cnt = sizeof(adv_uuids) / sizeof(adv_uuids[0]);
    srdata.uuids_complete.p_uuids  = adv_uuids;

    err_code = ble_advdata_encode(&srdata, m_adv_data.scan_rsp_data.p_data, &m_adv_data.scan_rsp_data.len);
    APP_ERROR_CHECK(err_code);

  

    // Set advertising parameters.
    memset(&adv_params, 0, sizeof(adv_params));

    adv_params.primary_phy     = BLE_GAP_PHY_1MBPS;
    adv_params.duration        = APP_ADV_DURATION;
    adv_params.properties.type = BLE_GAP_ADV_TYPE_CONNECTABLE_SCANNABLE_UNDIRECTED;

    adv_params.p_peer_addr     = NULL;
    adv_params.filter_policy   = BLE_GAP_ADV_FP_ANY;
    adv_params.interval        = APP_ADV_INTERVAL;

    err_code = sd_ble_gap_adv_set_configure(&m_adv_handle, &m_adv_data, &adv_params);
    APP_ERROR_CHECK(err_code);
   
}


/**@brief Function for Update the Advertising functionality.
 */
 void advertising_update(void)
{
   ret_code_t               err_code;
    uint16_t                 input_voltage;
    ble_advdata_t            advdata; //adv data
    ble_advdata_manuf_data_t manuf_data;
    ble_gap_adv_params_t     adv_params;

    (void) sd_ble_gap_adv_stop(m_adv_handle);
    

    advert_str[6]=0X0A;
    advert_str[7]=0X0A;
    advert_str[8]=0X0A;
    advert_str[9]=0X0A;
    advert_str[10]=0X0A;
    advert_str[11]=0X0A;
    advert_str[12]=0X0A;
    advert_str[13]=0X0A;
    advert_str[14]=0X0A;
    advert_str[15]=0X0A;
 
     // Build and set advertising data.
    memset(&advdata, 0, sizeof(advdata));

    manuf_data.company_identifier = 0x4341;
    manuf_data.data.p_data        = advert_str;
    manuf_data.data.size          = adv_str_len;

    advdata.name_type             = BLE_ADVDATA_FULL_NAME;
    advdata.include_appearance    = false;
    advdata.flags                 = BLE_GAP_ADV_FLAGS_LE_ONLY_GENERAL_DISC_MODE;
    advdata.p_manuf_specific_data = &manuf_data;

     err_code = ble_advdata_encode(&advdata, m_adv_data.adv_data.p_data, &m_adv_data.adv_data.len);
     APP_ERROR_CHECK(err_code);
    
   // Set advertising parameters.
    memset(&adv_params, 0, sizeof(adv_params));

    adv_params.primary_phy     = BLE_GAP_PHY_1MBPS;
    adv_params.duration        = APP_ADV_DURATION;
    if(conn_status==true)
    {
    //  adv_params.properties.type = BLE_GAP_ADV_TYPE_CONNECTABLE_SCANNABLE_UNDIRECTED;
      adv_params.properties.type = BLE_GAP_ADV_TYPE_NONCONNECTABLE_SCANNABLE_UNDIRECTED;
    }
    else
    {
      adv_params.properties.type = BLE_GAP_ADV_TYPE_CONNECTABLE_SCANNABLE_UNDIRECTED;
    }   
    adv_params.p_peer_addr     = NULL;
    adv_params.filter_policy   = BLE_GAP_ADV_FP_ANY;
    adv_params.interval        = APP_ADV_INTERVAL;

    err_code = sd_ble_gap_adv_set_configure(&m_adv_handle, &m_adv_data, &adv_params);
    APP_ERROR_CHECK(err_code);


}


void timer_handler(nrf_timer_event_t event_type, void * p_context)
{
   
}

/**@brief Function for connect the Time event with  
 *
 * @param[out] None
 */
void saadc_sampling_event_init(void)
{
    ret_code_t err_code;
    
    err_code = nrf_drv_ppi_init();
    APP_ERROR_CHECK(err_code);
    //Time config
    nrf_drv_timer_config_t timer_cfg = NRF_DRV_TIMER_DEFAULT_CONFIG;
    timer_cfg.bit_width = NRF_TIMER_BIT_WIDTH_32;
    err_code = nrf_drv_timer_init(&m_timer, &timer_cfg, timer_handler);
    APP_ERROR_CHECK(err_code);

    /* setup m_timer for compare event every 1ms */
    uint32_t ticks = nrf_drv_timer_ms_to_ticks(&m_timer, 1);
    nrf_drv_timer_extended_compare(&m_timer,
                                   NRF_TIMER_CC_CHANNEL0,
                                   ticks,
                                   NRF_TIMER_SHORT_COMPARE0_CLEAR_MASK,
                                   false);
    nrf_drv_timer_enable(&m_timer);

    uint32_t timer_compare_event_addr = nrf_drv_timer_compare_event_address_get(&m_timer,
                                                                                NRF_TIMER_CC_CHANNEL0);
    uint32_t saadc_sample_task_addr   = nrf_drv_saadc_sample_task_get();

    /* setup ppi channel so that timer compare event is triggering sample task in SAADC */
    err_code = nrf_drv_ppi_channel_alloc(&m_ppi_channel);
    APP_ERROR_CHECK(err_code);

    err_code = nrf_drv_ppi_channel_assign(m_ppi_channel,
                                          timer_compare_event_addr,
                                          saadc_sample_task_addr);
    APP_ERROR_CHECK(err_code);
}

void saadc_sampling_event_enable(void)
{
    ret_code_t err_code = nrf_drv_ppi_channel_enable(m_ppi_channel);

    APP_ERROR_CHECK(err_code);
}

/**@brief Function for initializing buttons and leds.
 *
 * @param[out] p_erase_bonds  Will be true if the clear bonding button was pressed to wake the application up.
 */
static void buttons_leds_init(bool * p_erase_bonds)
{
    ret_code_t err_code;
    bsp_event_t startup_event;

  //  err_code = bsp_init(BSP_INIT_LEDS | BSP_INIT_BUTTONS, bsp_event_handler);
    err_code = bsp_init(BSP_INIT_LEDS,bsp_event_handler);
    APP_ERROR_CHECK(err_code);

    //button init() no need 
   // err_code = bsp_btn_ble_init(NULL, &startup_event);
  //  APP_ERROR_CHECK(err_code);

    *p_erase_bonds = (startup_event == BSP_EVENT_CLEAR_BONDING_DATA);
}


/**@brief Function for initializing the nrf log module.
 */
static void log_init(void)
{
    ret_code_t err_code = NRF_LOG_INIT(NULL);
    APP_ERROR_CHECK(err_code);

    NRF_LOG_DEFAULT_BACKENDS_INIT();
}


/**@brief Function for initializing power management.
 */
static void power_management_init(void)
{
    ret_code_t err_code;
    err_code = nrf_pwr_mgmt_init();
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for handling the idle state (main loop).
 *
 * @details If there is no pending log operation, then sleep until next the next event occurs.
 */
static void idle_state_handle(void)
{
    if (NRF_LOG_PROCESS() == false)
    {
        nrf_pwr_mgmt_run();
    }
}


/**@brief Function for static passkey.
 */
 static void set_static_passkey()
 {
    static ble_opt_t    m_static_pin_option;
    
    uint8_t passkey[] = STATIC_PASSKEY;
    
    m_static_pin_option.gap_opt.passkey.p_passkey = &passkey[0];
    
    uint32_t err_code =  sd_ble_opt_set(BLE_GAP_OPT_PASSKEY, &m_static_pin_option);

    APP_ERROR_CHECK(err_code);
 
 }

/**@brief Function for starting advertising.
 */
static void advertising_start(bool erase_bonds)
{
    /*
   
   if(Pro_level==2)
   {
      ret_code_t err_code = ble_advertising_start(&m_advertising, BLE_ADV_MODE_FAST_NOCONNECT);
       APP_ERROR_CHECK(err_code);
   }
   else if(Pro_level==1)
   {
      ret_code_t err_code =ble_advertising_start(&m_advertising, BLE_ADV_MODE_FAST);
       APP_ERROR_CHECK(err_code);
   }
   */
  ret_code_t           err_code;
  err_code = sd_ble_gap_adv_start(m_adv_handle, APP_BLE_CONN_CFG_TAG);

   APP_ERROR_CHECK(err_code);

}


/**@brief Function for get mac address.
 */
static void getMACaddress()
{
   ble_gap_addr_t MAC_address;
   sd_ble_gap_addr_get(&MAC_address);
  // if(Pro_level==1)
  // {
      advert_str[0]=MAC_address.addr[5];
      advert_str[1]=MAC_address.addr[4];
      advert_str[2]=MAC_address.addr[3];
      advert_str[3]=MAC_address.addr[2];
      advert_str[4]=MAC_address.addr[1];
      advert_str[5]=MAC_address.addr[0];
      advert_str[6]=0x00; 
      advert_str[7]=0x00;
      advert_str[8]=0x00;
      advert_str[9]=0x00;
      advert_str[10]=0x00;
      advert_str[11]=0x00;
      advert_str[12]=powerstatus;
      /*
      advert_str[13]=FW_version_val1;
      advert_str[14]=FW_version_val2;
      advert_str[15]=FW_version_val3;
      */
      advert_str[13]=softver_value[2];
      advert_str[14]=softver_value[1];
      advert_str[15]=softver_value[0];
 //  }
  
   



}



/**@brief Function for dealing the BLE command .
 */
void BLE_cmd_read_send()
{
    uint32_t err_code;
    uint32_t write_data;
    uint32_t verify_data;
  //  uint8_t Data1=0;
  //  uint8_t Data2=0;
  //  uint8_t Data3=0;
  //  uint8_t Data4=0;
    
   uint16_t length = 0;
   uint8_t SenddataArry[7] = {0x81,0XFF,0X11,0X11,0X11,0X11,0XED};
   if(Rec_flag==true)
   {
      switch(AQS_Rev_Data[5])
      {
        case TestRead_SerialNum:  
             length=7; 
             break;
        case TestRead_SoftwareVer:    
             length=7;
             break;
        default:
            break;  
      }       
      do
        {
          //err_code = ble_nus_data_send(&m_nus,  AQS_Send_Data.Data,  &length, m_conn_handle);
        //  NRF_LOG_INFO("m_conn_handle%d\n",m_conn_handle);
          err_code = ble_nus_data_send(&m_nus,  SenddataArry,  &length, m_conn_handle);
          if ((err_code != NRF_ERROR_INVALID_STATE) &&
              (err_code != NRF_ERROR_RESOURCES) &&
              (err_code != NRF_ERROR_NOT_FOUND))
           {
              APP_ERROR_CHECK(err_code);
           }
        }while (err_code == NRF_ERROR_RESOURCES);
       Rec_flag=false;
       NRF_LOG_INFO("Send back the data to centrol board successfully.");
   }
 
}




int main(void)
{
    bool erase_bonds;
    ret_code_t rc;
    uint32_t err_code = NRF_LOG_INIT(NULL);
    APP_ERROR_CHECK(err_code);

    NRF_LOG_DEFAULT_BACKENDS_INIT();

    ret_code_t ret_code = nrf_pwr_mgmt_init();
    APP_ERROR_CHECK(ret_code);

    // Initialize.
    
    timers_init();
    bsp_demo_init();
    saadc_sampling_event_init();
    saadc_sampling_event_enable();
    // Start execution.
    uart_init();
    ble_stack_init();
    set_static_passkey();
    gap_params_init();
    gatt_init();
    getMACaddress();
   
   
    services_init();
    advertising_init();
    conn_params_init();
    peer_manager_init();

    err_code = sd_ble_gap_tx_power_set(BLE_GAP_TX_POWER_ROLE_ADV, m_adv_handle,-8);
    APP_ERROR_CHECK(err_code);
   
    advertising_start(true);
  
    NRF_LOG_INFO("AQS started.");
    while (1)
    {
       BLE_cmd_read_send();
       if(Timecount>1000)
       {
          advertising_update();
          advertising_start(true);
          Timecount=0;
       }
        idle_state_handle();
         NRF_LOG_FLUSH();
    }
}


/**
 * @}
 */
